// File          : InvalidHueValueException.java
// Date          : 20th March 2007
// Last Modified : 13th March 2008
// Author        : Nicola L.C. Talbot
//                 http://theoval.cmp.uea.ac.uk/~nlct/

/*
    Copyright (C) 2006 Nicola L.C. Talbot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

package uk.ac.uea.cmp.nlct.jdr;

// error.invalid_hue
/**
 * Exception thrown when an invalid hue value is used.
 * @author Nicola L C Talbot
 */
public class InvalidHueValueException extends InvalidFormatException
{
   /**
    * Initialises indicating the invalid value.
    * @param invalidValue invalid hue value
    */
   public InvalidHueValueException(double invalidValue)
   {
      super("Invalid hue value "+invalidValue);

      invalidValue_ = invalidValue;
   }

   /**
    * Initialises indicating the invalid value and the line number
    * where the error occurred.
    * @param invalidValue invalid hue value
    * @param line line number
    */
   public InvalidHueValueException(double invalidValue, int line)
   {
      super("Invalid hue value "+invalidValue+" on line "+line);

      invalidValue_ = invalidValue;
      lineNum = line;
   }

   /**
    * Gets the invalid value.
    * @return invalid hue value
    */
   public double getInvalidValue()
   {
      return invalidValue_;
   }

   /**
    * Gets the line number.
    * @return line number or -1 if none specified
    */
   public int getLineNum()
   {
      return lineNum;
   }

   private double invalidValue_;
   private int lineNum = -1;
}
