// File          : JDRPartialLineLoaderListener.java
// Date          : 18th August 2010
// Last Modified : 18th August 2010
// Author        : Nicola L.C. Talbot
//                 http://theoval.cmp.uea.ac.uk/~nlct/

/*
    Copyright (C) 2006 Nicola L.C. Talbot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/

package uk.ac.uea.cmp.nlct.jdr;

import java.io.*;
import java.util.*;

import uk.ac.uea.cmp.nlct.jdr.io.*;

/**
 * Loader listener for partial line segments.
 * @author Nicola L C Talbot
 */

public class JDRPartialLineLoaderListener extends JDRSegmentLoaderListener
{
   /**
    * Gets the character identifying the partial line segment type
    * in JDR format.
    * @param version the JDR version number
    * @return JDR segment id
    */
   public char getJDRid(float version)
   {
      return 'l';
   }

   /**
    * Reads partial line specified in JDR format. (Doesn't include the
    * segment ID returned by {@link #getJDRid(float)} which is dealt
    * with by the segment loader.)
    * @param din the input stream
    * @param version the JDR version number
    * @throws IOException if I/O error occurs
    * @throws InvalidFormatException if there is something wrong
    * with the segment format
    * @return the line segment read from the input stream
    */
   public JDRObject readJDR(DataInputStream din, float version)
   throws IOException,InvalidFormatException
   {
      return new JDRPartialLine(null, null);
   }

   public void writeJDR(JDRObject object, DataOutputStream dout,
      float version)
      throws IOException, InvalidFormatException
   {
   }

   /**
    * Gets the character identifying the line segment type
    * in AJR format.
    * @param version the AJR version number
    * @return AJR segment id
    */
   public char getAJRid(float version)
   {
      return 'l';
   }

   /**
    * Reads partial line specified in AJR format. (Doesn't include the
    * segment ID returned by {@link #getAJRid(float)} which is dealt
    * with by the segment loader.)
    * @param in the input stream
    * @param version the AJR version number
    * @return the line segment read from the input stream
    * @throws IOException if I/O error occurs
    * @throws InvalidFormatException if there is something wrong
    * with the segment format
    * @throws java.nio.BufferOverflowException if the {@link uk.ac.uea.cmp.nlct.jdr.io.AJR}
    * buffer overflows
    * @throws EOFException if the end of file occurs unexpectedly
    */
   public JDRObject readAJR(BufferedReader in, float version)
   throws IOException,InvalidFormatException,
          java.nio.BufferOverflowException,
          EOFException
   {
      return new JDRPartialLine(null, null);
   }

   public void writeAJR(JDRObject object, PrintWriter out, float version)
      throws IOException, InvalidFormatException
   {
   }
}
