// File          : EPSPutInterval.java
// Purpose       : class representing putinterval operator
// Date          : 1st June 2008
// Last Modified : 1st June 2008
// Author        : Nicola L.C. Talbot
//               http://theoval.cmp.uea.ac.uk/~nlct/

/*
    Copyright (C) 2006 Nicola L.C. Talbot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package uk.ac.uea.cmp.nlct.jdr.io.eps.operators;

import java.io.*;
import java.awt.geom.*;

import uk.ac.uea.cmp.nlct.jdr.*;
import uk.ac.uea.cmp.nlct.jdr.io.eps.*;
import uk.ac.uea.cmp.nlct.jdr.io.EPS;

/**
 * Object representing putinterval operator.
 * @author Nicola L C Talbot
 */
public class EPSPutInterval extends EPSOperator
{
   public EPSPutInterval()
   {
      super("putinterval");
   }

   public void execute(EPSStack stack, EPS eps)
      throws InvalidFormatException,
             NoninvertibleTransformException,
             IOException
   {
      EPSIndexComposite interval = stack.popEPSIndexComposite();
      int index = stack.popInteger();
      EPSIndexComposite object = stack.popEPSIndexComposite();

      try
      {
         object.putInterval(index, interval);
      }
      catch (ArrayIndexOutOfBoundsException e)
      {
         throw new InvalidFormatException(
            "(putinterval) index out of bounds", eps.getLineNum());
      }
      catch (NoReadAccessException e)
      {
         throw new NoReadAccessException(
            "(putinterval) no read access", eps.getLineNum());
      }
      catch (NoWriteAccessException e)
      {
         throw new NoWriteAccessException(
            "(putinterval) no write access", eps.getLineNum());
      }
   }

}
