// File          : Jdr2Png.java
// Description   : command line jdr to png converter
// Date          : 5th June 2008
// Last Modified : 1 October 2009
// Author        : Nicola L C Talbot
//                 http://theoval.cmp.uea.ac.uk/~nlct/

/*
    Copyright (C) 2006 Nicola L.C. Talbot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package uk.ac.uea.cmp.nlct.jdr2png;

import java.io.*;
import uk.ac.uea.cmp.nlct.jdr.*;
import uk.ac.uea.cmp.nlct.jdr.io.*;

/**
 * Command line JDR to PNG converter.
 * @author Nicola L C Talbot
 */
public class Jdr2Png
{
   public static void appVersion()
   {
      System.err.println("jdr2png 1.5");
      System.err.println("Copyright (C) 2007 Nicola L C Talbot");
      System.err.println("This is free software distributed under the GNU General Public License.");
      System.err.println("There is NO WARRANTY. See accompanying licence file for details.");
   }

   public static void syntax(int exitCode)
   {
      System.err.println("Syntax: jdr2png [-version] <jdr file> <png file>");
      System.exit(exitCode);
   }

   public static void main(String[] args)
   {
      String jdrFile = null;
      String pngFile = null;

      JDR.messageSystem.suspend();

      for (int i = 0; i < args.length; i++)
      {
         if (args[i].equals("-version"))
         {
            appVersion();
            System.exit(0);
         }
         else if (args[i].equals("-help"))
         {
            syntax(0);
         }
         else if (args[i].startsWith("-"))
         {
            System.err.println("Unknown switch '"+args[i]+"'");
            syntax(1);
         }
         else if (jdrFile == null)
         {
            jdrFile = args[i];
         }
         else if (pngFile == null)
         {
            pngFile = args[i];
         }
         else
         {
            System.err.println("Too many file names specified");
            syntax(1);
         }
      }

      if (jdrFile == null)
      {
         System.err.println("Missing input file");
         syntax(1);
      }
      else if (pngFile == null)
      {
         System.err.println("Missing output file");
         syntax(1);
      }


      DataInputStream in=null;

      try
      {
         in = new DataInputStream(new FileInputStream(jdrFile));
      }
      catch (IOException e)
      {
         System.err.println("Unable to open "
            +e.getMessage());
         System.exit(1);
      }
      catch (Exception e)
      {
         System.err.println(e);
         System.exit(1);
      }

      String fileVersion = "";
      CanvasSettings settings = new CanvasSettings();
      JDRGroup paths=null;

      try
      {
         paths = JDR.load(in, settings);
      }
      catch (InvalidFormatException e)
      {
         System.err.println(e.getMessage());
         System.exit(1);
      }
      catch (EOFException e)
      {
         System.err.println("Unexpected EOF. (File corrupt?)");
         System.exit(1);
      }
      catch (Exception e)
      {
         System.err.println(e);
         System.exit(1);
      }

      try
      {
         in.close();
      }
      catch (Exception e)
      {
         System.err.println(e);
         System.exit(1);
      }


      try
      {
         PNG.save(paths, pngFile);
      }
      catch (Exception e)
      {
         System.err.println(e);
         System.exit(1);
      }
   }
}
