// File          : IntRangeDocument.java
// Description   : Provide document that only allow integer values in a given range
// Date          : 6th February 2006
// Last Modified : 7th June 2008
// Author        : Nicola L.C. Talbot
//                 http://theoval.cmp.uea.ac.uk/~nlct/

/*
    Copyright (C) 2006 Nicola L.C. Talbot

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/
package uk.ac.uea.cmp.nlct.jdrresources.numfield;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;

/**
 * Document that only allows integer numbers in a given range.
 * @author Nicola L C Talbot
 */
public class IntRangeDocument extends PlainDocument
{
   /**
    * Initialises with the given minimum and maximum values.
    * @param minValue the minimum value
    * @param maxValue the maximum value
    */
   public IntRangeDocument(int minValue, int maxValue)
   {
      min = minValue;
      max = maxValue;
   }

   public void insertString(int offs, String str, AttributeSet a)
      throws BadLocationException
   {
      if (str == null) return;

      String oldString = getText(0, getLength());
      String newString = oldString.substring(0,offs)
                       +str+oldString.substring(offs);
      try
      {
         int n = Integer.parseInt(newString);
         if (n >= min && n <= max)
         {
            super.insertString(offs, str, a);
         }
      }
      catch (NumberFormatException e)
      {
         if (min < 0 && newString.equals("-"))
         {
            super.insertString(offs, str, a);
         }
      }
   }

   private int min, max;
}
